package dev.pfaff.glass;

import org.jspecify.annotations.Nullable;

/**
 * CPU architecture information.
 */
public sealed interface Arch {
	Arch HOST = switch (System.getProperty("os.arch")) {
		case null -> new Unknown(null);
		case "x86", "x86_32", "i386" -> new x86();
		case "x86_64", "amd64" -> new amd64();
		case "aarch64" -> new aarch64();
		case String name -> new Unknown(name);
	};

	record x86() implements Arch {}

	record amd64() implements Arch {}

	record aarch64() implements Arch {}

	record Unknown(@Nullable String name) implements Arch {}
}
