package dev.pfaff.glass.opengl;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.StandardCharsets;

public record ConstPointer<T>(MemorySegment ms) implements Pointer<T> {
	public ConstPointer {
		if (ms.equals(MemorySegment.NULL)) throw new NullPointerException("Attempted non-nullable construction of a ConstPointer with NULL address");
		ms = ms.asReadOnly();
	}

	@Override
	public ConstPointer<T> asConst() {
		return this;
	}

	public static <T> ConstPointer<T> newNullable(MemorySegment ms) {
		return ms.equals(MemorySegment.NULL) ? null : new ConstPointer<>(ms);
	}

	public static ConstPointer<Byte> allocateCString(Arena arena, String string) {
		var ms = arena.allocate(string.length()+1);
		var encoder = StandardCharsets.US_ASCII.newEncoder();
		var buf = ms.asByteBuffer();
		var result = encoder.encode(CharBuffer.wrap(string), buf, true);
		if (!result.isUnderflow()) {
			try {
				result.throwException();
			} catch (CharacterCodingException e) {
				throw new RuntimeException(e);
			}
			// if there is not another error,
			// we should always underflow because we need room for the NUL byte.
			throw new AssertionError();
		}
		buf.put((byte)0);
		return new ConstPointer<>(ms.asSlice(0, buf.position()));
	}
}
