package dev.pfaff.glass.opengl;

import dev.pfaff.log4truth.Logger;
import dev.pfaff.glass.opengl.enums.GLErrorCode;
import dev.pfaff.glass.opengl.enums.GLGetPName;
import dev.pfaff.glass.opengl.enums.GLStringName;

import static java.nio.charset.StandardCharsets.US_ASCII;

public final class GLDebugUtil {
	public static void logGLInfo(GL gl) {
		var vendor = gl.glGetString(GLStringName.VENDOR).ms().reinterpret(Long.MAX_VALUE).getString(0, US_ASCII);
		var renderer = gl.glGetString(GLStringName.RENDERER).ms().reinterpret(Long.MAX_VALUE).getString(0, US_ASCII);
		var version = gl.glGetString(GLStringName.VERSION).ms().reinterpret(Long.MAX_VALUE).getString(0, US_ASCII);
		var glslVersion =
				gl.glGetString(GLStringName.SHADING_LANGUAGE_VERSION).ms().reinterpret(Long.MAX_VALUE).getString(
				0,
				US_ASCII);
		var extensions = new StringBuilder();
		for (int i = 0; ; i++) {
			ConstPointer<Byte> ptr;
			try {
				ptr = gl.glGetStringi(GLStringName.EXTENSIONS, i);
			} catch (GLException e) {
				// end of "array"
				if (e.code == GLErrorCode.INVALID_VALUE) break;
				throw e;
			}
			if (ptr == null) break;
			var s = ptr.ms().reinterpret(Long.MAX_VALUE).getString(0, US_ASCII);
			if (!extensions.isEmpty()) {
				extensions.ensureCapacity(2 + s.length());
				extensions.append(", ");
			}
			extensions.append(s);
		}

		var maxTextureSize = gl.glGetInteger(GLGetPName.MAX_TEXTURE_SIZE);
		var maxTextureLayers = gl.glGetInteger(GLGetPName.MAX_ARRAY_TEXTURE_LAYERS);
		var max3dTextureSize = gl.glGetInteger(GLGetPName.MAX_3D_TEXTURE_SIZE);

		Logger.log(() -> STR."""
			GL Info:
			  Vendor: \{vendor}
			  Renderer: \{renderer}
			  Version: \{version}
			  Extensions: \{extensions}
			  GLSL Version: \{glslVersion}

			  Limits:
			    Max Texture Size: \{maxTextureSize}
			    Max Texture Layers: \{maxTextureLayers}
			    Max 3D Texture Size: \{max3dTextureSize}""");
	}
}
