package dev.pfaff.glass.opengl;

import dev.pfaff.glass.opengl.enums.GLErrorCode;

public class GLException extends RuntimeException {
	public final GLErrorCode code;

	public GLException(GLErrorCode code) {
		this(code, makeErrorMessage(code), null);
	}

	public GLException(GLErrorCode code, String message) {
		this(code, makeErrorMessage(code) + " " + message, null);
	}

	private GLException(GLErrorCode code, String message, StackTraceElement[] stackTrace) {
		super(message);
		this.code = code;
		if (stackTrace != null) setStackTrace(stackTrace);
	}

	/**
	 * @param message the additional message. It <em>should</em> end with a {@code '.'}.
	 */
	public final GLException withAdditionalMessage(String message) {
		return new GLException(code, this.getMessage() + " " + message, this.getStackTrace());
	}

	private static String makeErrorMessage(GLErrorCode code) {
		return STR."Error code \{code.glEnum()} (\{code}): \{describeError(code)}";
	}

	public static String describeError(GLErrorCode code) {
		return switch (code) {
			case NO_ERROR ->
					"No error has been recorded. The developer should not be trying to get a description for a " +
							"non-error.";
			case INVALID_ENUM ->
					"An unacceptable value is specified for an enumerated argument. The offending command is ignored " +
							"and has no other side effect than to set the error flag.";
			case INVALID_VALUE ->
					"A numeric argument is out of range. The offending command is ignored and has no other side effect" +
							" than to set the error flag.";
			case INVALID_OPERATION ->
					"The specified operation is not allowed in the current state. The offending command is ignored and" +
							" has no other side effect than to set the error flag.";
			case INVALID_FRAMEBUFFER_OPERATION ->
					"The framebuffer object is not complete. The offending command is ignored and has no other side " +
							"effect than to set the error flag.";
			case OUT_OF_MEMORY ->
					"There is not enough memory left to execute the command. The state of the GL is undefined, except " +
							"for the state of the error flags, after this error is recorded.";
			case STACK_UNDERFLOW ->
					"An attempt has been made to perform an operation that would cause an internal stack to " +
							"underflow.";
			case STACK_OVERFLOW ->
					"An attempt has been made to perform an operation that would cause an internal stack to overflow.";
			case TABLE_TOO_LARGE_EXT -> "The specified table is too large.";
			case TEXTURE_TOO_LARGE_EXT -> "The specified texture is too large.";
		};
	}
}
