package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLContextFlagMask implements GLEnum {
	CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT(0x1),
	CONTEXT_FLAG_DEBUG_BIT(0x2),
	CONTEXT_FLAG_ROBUST_ACCESS_BIT(0x4),
	CONTEXT_FLAG_NO_ERROR_BIT(0x8),
	CONTEXT_FLAG_PROTECTED_CONTENT_BIT_EXT(0x10);
	
	private final int glEnum;

	GLContextFlagMask(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLContextFlagMask> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLContextFlagMask fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT;
			case 0x2 -> CONTEXT_FLAG_DEBUG_BIT;
			case 0x4 -> CONTEXT_FLAG_ROBUST_ACCESS_BIT;
			case 0x8 -> CONTEXT_FLAG_NO_ERROR_BIT;
			case 0x10 -> CONTEXT_FLAG_PROTECTED_CONTENT_BIT_EXT;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLContextFlagMask");
		};
	}
}