package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLMaterialParameter implements GLEnum {
	AMBIENT(0x1200),
	DIFFUSE(0x1201),
	SPECULAR(0x1202),
	EMISSION(0x1600),
	SHININESS(0x1601),
	AMBIENT_AND_DIFFUSE(0x1602),
	COLOR_INDEXES(0x1603);
	
	private final int glEnum;

	GLMaterialParameter(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLMaterialParameter> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLMaterialParameter fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1200 -> AMBIENT;
			case 0x1201 -> DIFFUSE;
			case 0x1202 -> SPECULAR;
			case 0x1600 -> EMISSION;
			case 0x1601 -> SHININESS;
			case 0x1602 -> AMBIENT_AND_DIFFUSE;
			case 0x1603 -> COLOR_INDEXES;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLMaterialParameter");
		};
	}
}