package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLPathCoordType implements GLEnum {
	CLOSE_PATH_NV(0x0),
	MOVE_TO_NV(0x2),
	RELATIVE_MOVE_TO_NV(0x3),
	LINE_TO_NV(0x4),
	RELATIVE_LINE_TO_NV(0x5),
	HORIZONTAL_LINE_TO_NV(0x6),
	RELATIVE_HORIZONTAL_LINE_TO_NV(0x7),
	VERTICAL_LINE_TO_NV(0x8),
	RELATIVE_VERTICAL_LINE_TO_NV(0x9),
	QUADRATIC_CURVE_TO_NV(0xa),
	RELATIVE_QUADRATIC_CURVE_TO_NV(0xb),
	CUBIC_CURVE_TO_NV(0xc),
	RELATIVE_CUBIC_CURVE_TO_NV(0xd),
	SMOOTH_QUADRATIC_CURVE_TO_NV(0xe),
	RELATIVE_SMOOTH_QUADRATIC_CURVE_TO_NV(0xf),
	SMOOTH_CUBIC_CURVE_TO_NV(0x10),
	RELATIVE_SMOOTH_CUBIC_CURVE_TO_NV(0x11),
	SMALL_CCW_ARC_TO_NV(0x12),
	RELATIVE_SMALL_CCW_ARC_TO_NV(0x13),
	SMALL_CW_ARC_TO_NV(0x14),
	RELATIVE_SMALL_CW_ARC_TO_NV(0x15),
	LARGE_CCW_ARC_TO_NV(0x16),
	RELATIVE_LARGE_CCW_ARC_TO_NV(0x17),
	LARGE_CW_ARC_TO_NV(0x18),
	RELATIVE_LARGE_CW_ARC_TO_NV(0x19),
	CONIC_CURVE_TO_NV(0x1a),
	RELATIVE_CONIC_CURVE_TO_NV(0x1b),
	ROUNDED_RECT_NV(0xe8),
	RELATIVE_ROUNDED_RECT_NV(0xe9),
	ROUNDED_RECT2_NV(0xea),
	RELATIVE_ROUNDED_RECT2_NV(0xeb),
	ROUNDED_RECT4_NV(0xec),
	RELATIVE_ROUNDED_RECT4_NV(0xed),
	ROUNDED_RECT8_NV(0xee),
	RELATIVE_ROUNDED_RECT8_NV(0xef),
	RESTART_PATH_NV(0xf0),
	DUP_FIRST_CUBIC_CURVE_TO_NV(0xf2),
	DUP_LAST_CUBIC_CURVE_TO_NV(0xf4),
	RECT_NV(0xf6),
	RELATIVE_RECT_NV(0xf7),
	CIRCULAR_CCW_ARC_TO_NV(0xf8),
	CIRCULAR_CW_ARC_TO_NV(0xfa),
	CIRCULAR_TANGENT_ARC_TO_NV(0xfc),
	ARC_TO_NV(0xfe),
	RELATIVE_ARC_TO_NV(0xff);
	
	private final int glEnum;

	GLPathCoordType(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLPathCoordType> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLPathCoordType fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x0 -> CLOSE_PATH_NV;
			case 0x2 -> MOVE_TO_NV;
			case 0x3 -> RELATIVE_MOVE_TO_NV;
			case 0x4 -> LINE_TO_NV;
			case 0x5 -> RELATIVE_LINE_TO_NV;
			case 0x6 -> HORIZONTAL_LINE_TO_NV;
			case 0x7 -> RELATIVE_HORIZONTAL_LINE_TO_NV;
			case 0x8 -> VERTICAL_LINE_TO_NV;
			case 0x9 -> RELATIVE_VERTICAL_LINE_TO_NV;
			case 0xa -> QUADRATIC_CURVE_TO_NV;
			case 0xb -> RELATIVE_QUADRATIC_CURVE_TO_NV;
			case 0xc -> CUBIC_CURVE_TO_NV;
			case 0xd -> RELATIVE_CUBIC_CURVE_TO_NV;
			case 0xe -> SMOOTH_QUADRATIC_CURVE_TO_NV;
			case 0xf -> RELATIVE_SMOOTH_QUADRATIC_CURVE_TO_NV;
			case 0x10 -> SMOOTH_CUBIC_CURVE_TO_NV;
			case 0x11 -> RELATIVE_SMOOTH_CUBIC_CURVE_TO_NV;
			case 0x12 -> SMALL_CCW_ARC_TO_NV;
			case 0x13 -> RELATIVE_SMALL_CCW_ARC_TO_NV;
			case 0x14 -> SMALL_CW_ARC_TO_NV;
			case 0x15 -> RELATIVE_SMALL_CW_ARC_TO_NV;
			case 0x16 -> LARGE_CCW_ARC_TO_NV;
			case 0x17 -> RELATIVE_LARGE_CCW_ARC_TO_NV;
			case 0x18 -> LARGE_CW_ARC_TO_NV;
			case 0x19 -> RELATIVE_LARGE_CW_ARC_TO_NV;
			case 0x1a -> CONIC_CURVE_TO_NV;
			case 0x1b -> RELATIVE_CONIC_CURVE_TO_NV;
			case 0xe8 -> ROUNDED_RECT_NV;
			case 0xe9 -> RELATIVE_ROUNDED_RECT_NV;
			case 0xea -> ROUNDED_RECT2_NV;
			case 0xeb -> RELATIVE_ROUNDED_RECT2_NV;
			case 0xec -> ROUNDED_RECT4_NV;
			case 0xed -> RELATIVE_ROUNDED_RECT4_NV;
			case 0xee -> ROUNDED_RECT8_NV;
			case 0xef -> RELATIVE_ROUNDED_RECT8_NV;
			case 0xf0 -> RESTART_PATH_NV;
			case 0xf2 -> DUP_FIRST_CUBIC_CURVE_TO_NV;
			case 0xf4 -> DUP_LAST_CUBIC_CURVE_TO_NV;
			case 0xf6 -> RECT_NV;
			case 0xf7 -> RELATIVE_RECT_NV;
			case 0xf8 -> CIRCULAR_CCW_ARC_TO_NV;
			case 0xfa -> CIRCULAR_CW_ARC_TO_NV;
			case 0xfc -> CIRCULAR_TANGENT_ARC_TO_NV;
			case 0xfe -> ARC_TO_NV;
			case 0xff -> RELATIVE_ARC_TO_NV;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLPathCoordType");
		};
	}
}