package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLPixelStoreParameter implements GLEnum {
	UNPACK_SWAP_BYTES(0xcf0),
	UNPACK_LSB_FIRST(0xcf1),
	UNPACK_ROW_LENGTH(0xcf2),
	UNPACK_SKIP_ROWS(0xcf3),
	UNPACK_SKIP_PIXELS(0xcf4),
	UNPACK_ALIGNMENT(0xcf5),
	PACK_SWAP_BYTES(0xd00),
	PACK_LSB_FIRST(0xd01),
	PACK_ROW_LENGTH(0xd02),
	PACK_SKIP_ROWS(0xd03),
	PACK_SKIP_PIXELS(0xd04),
	PACK_ALIGNMENT(0xd05),
	PACK_SKIP_IMAGES(0x806b),
	PACK_IMAGE_HEIGHT(0x806c),
	UNPACK_SKIP_IMAGES(0x806d),
	UNPACK_IMAGE_HEIGHT(0x806e),
	PACK_SKIP_VOLUMES_SGIS(0x8130),
	PACK_IMAGE_DEPTH_SGIS(0x8131),
	UNPACK_SKIP_VOLUMES_SGIS(0x8132),
	UNPACK_IMAGE_DEPTH_SGIS(0x8133),
	PIXEL_TILE_WIDTH_SGIX(0x8140),
	PIXEL_TILE_HEIGHT_SGIX(0x8141),
	PIXEL_TILE_GRID_WIDTH_SGIX(0x8142),
	PIXEL_TILE_GRID_HEIGHT_SGIX(0x8143),
	PIXEL_TILE_GRID_DEPTH_SGIX(0x8144),
	PIXEL_TILE_CACHE_SIZE_SGIX(0x8145),
	PACK_RESAMPLE_SGIX(0x842e),
	UNPACK_RESAMPLE_SGIX(0x842f),
	PACK_SUBSAMPLE_RATE_SGIX(0x85a0),
	UNPACK_SUBSAMPLE_RATE_SGIX(0x85a1),
	PACK_RESAMPLE_OML(0x8984),
	UNPACK_RESAMPLE_OML(0x8985);
	
	private final int glEnum;

	GLPixelStoreParameter(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLPixelStoreParameter> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLPixelStoreParameter fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0xcf0 -> UNPACK_SWAP_BYTES;
			case 0xcf1 -> UNPACK_LSB_FIRST;
			case 0xcf2 -> UNPACK_ROW_LENGTH;
			case 0xcf3 -> UNPACK_SKIP_ROWS;
			case 0xcf4 -> UNPACK_SKIP_PIXELS;
			case 0xcf5 -> UNPACK_ALIGNMENT;
			case 0xd00 -> PACK_SWAP_BYTES;
			case 0xd01 -> PACK_LSB_FIRST;
			case 0xd02 -> PACK_ROW_LENGTH;
			case 0xd03 -> PACK_SKIP_ROWS;
			case 0xd04 -> PACK_SKIP_PIXELS;
			case 0xd05 -> PACK_ALIGNMENT;
			case 0x806b -> PACK_SKIP_IMAGES;
			case 0x806c -> PACK_IMAGE_HEIGHT;
			case 0x806d -> UNPACK_SKIP_IMAGES;
			case 0x806e -> UNPACK_IMAGE_HEIGHT;
			case 0x8130 -> PACK_SKIP_VOLUMES_SGIS;
			case 0x8131 -> PACK_IMAGE_DEPTH_SGIS;
			case 0x8132 -> UNPACK_SKIP_VOLUMES_SGIS;
			case 0x8133 -> UNPACK_IMAGE_DEPTH_SGIS;
			case 0x8140 -> PIXEL_TILE_WIDTH_SGIX;
			case 0x8141 -> PIXEL_TILE_HEIGHT_SGIX;
			case 0x8142 -> PIXEL_TILE_GRID_WIDTH_SGIX;
			case 0x8143 -> PIXEL_TILE_GRID_HEIGHT_SGIX;
			case 0x8144 -> PIXEL_TILE_GRID_DEPTH_SGIX;
			case 0x8145 -> PIXEL_TILE_CACHE_SIZE_SGIX;
			case 0x842e -> PACK_RESAMPLE_SGIX;
			case 0x842f -> UNPACK_RESAMPLE_SGIX;
			case 0x85a0 -> PACK_SUBSAMPLE_RATE_SGIX;
			case 0x85a1 -> UNPACK_SUBSAMPLE_RATE_SGIX;
			case 0x8984 -> PACK_RESAMPLE_OML;
			case 0x8985 -> UNPACK_RESAMPLE_OML;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLPixelStoreParameter");
		};
	}
}