package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLQueryTarget implements GLEnum {
	TRANSFORM_FEEDBACK_OVERFLOW(0x82ec),
	VERTICES_SUBMITTED(0x82ee),
	PRIMITIVES_SUBMITTED(0x82ef),
	VERTEX_SHADER_INVOCATIONS(0x82f0),
	TIME_ELAPSED(0x88bf),
	SAMPLES_PASSED(0x8914),
	ANY_SAMPLES_PASSED(0x8c2f),
	PRIMITIVES_GENERATED(0x8c87),
	TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN(0x8c88),
	ANY_SAMPLES_PASSED_CONSERVATIVE(0x8d6a);
	
	private final int glEnum;

	GLQueryTarget(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLQueryTarget> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLQueryTarget fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x82ec -> TRANSFORM_FEEDBACK_OVERFLOW;
			case 0x82ee -> VERTICES_SUBMITTED;
			case 0x82ef -> PRIMITIVES_SUBMITTED;
			case 0x82f0 -> VERTEX_SHADER_INVOCATIONS;
			case 0x88bf -> TIME_ELAPSED;
			case 0x8914 -> SAMPLES_PASSED;
			case 0x8c2f -> ANY_SAMPLES_PASSED;
			case 0x8c87 -> PRIMITIVES_GENERATED;
			case 0x8c88 -> TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN;
			case 0x8d6a -> ANY_SAMPLES_PASSED_CONSERVATIVE;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLQueryTarget");
		};
	}
}