package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLTextureTarget implements GLEnum {
	TEXTURE_1D(0xde0),
	TEXTURE_2D(0xde1),
	PROXY_TEXTURE_1D(0x8063),
	PROXY_TEXTURE_2D(0x8064),
	TEXTURE_3D(0x806f),
	PROXY_TEXTURE_3D(0x8070),
	DETAIL_TEXTURE_2D_SGIS(0x8095),
	TEXTURE_4D_SGIS(0x8134),
	PROXY_TEXTURE_4D_SGIS(0x8135),
	TEXTURE_RECTANGLE(0x84f5),
	PROXY_TEXTURE_RECTANGLE(0x84f7),
	TEXTURE_CUBE_MAP(0x8513),
	TEXTURE_CUBE_MAP_POSITIVE_X(0x8515),
	TEXTURE_CUBE_MAP_NEGATIVE_X(0x8516),
	TEXTURE_CUBE_MAP_POSITIVE_Y(0x8517),
	TEXTURE_CUBE_MAP_NEGATIVE_Y(0x8518),
	TEXTURE_CUBE_MAP_POSITIVE_Z(0x8519),
	TEXTURE_CUBE_MAP_NEGATIVE_Z(0x851a),
	PROXY_TEXTURE_CUBE_MAP(0x851b),
	TEXTURE_1D_ARRAY(0x8c18),
	PROXY_TEXTURE_1D_ARRAY(0x8c19),
	TEXTURE_2D_ARRAY(0x8c1a),
	PROXY_TEXTURE_2D_ARRAY(0x8c1b),
	TEXTURE_BUFFER(0x8c2a),
	RENDERBUFFER(0x8d41),
	TEXTURE_CUBE_MAP_ARRAY(0x9009),
	PROXY_TEXTURE_CUBE_MAP_ARRAY(0x900b),
	TEXTURE_2D_MULTISAMPLE(0x9100),
	PROXY_TEXTURE_2D_MULTISAMPLE(0x9101),
	TEXTURE_2D_MULTISAMPLE_ARRAY(0x9102),
	PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY(0x9103);
	
	private final int glEnum;

	GLTextureTarget(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLTextureTarget> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLTextureTarget fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0xde0 -> TEXTURE_1D;
			case 0xde1 -> TEXTURE_2D;
			case 0x8063 -> PROXY_TEXTURE_1D;
			case 0x8064 -> PROXY_TEXTURE_2D;
			case 0x806f -> TEXTURE_3D;
			case 0x8070 -> PROXY_TEXTURE_3D;
			case 0x8095 -> DETAIL_TEXTURE_2D_SGIS;
			case 0x8134 -> TEXTURE_4D_SGIS;
			case 0x8135 -> PROXY_TEXTURE_4D_SGIS;
			case 0x84f5 -> TEXTURE_RECTANGLE;
			case 0x84f7 -> PROXY_TEXTURE_RECTANGLE;
			case 0x8513 -> TEXTURE_CUBE_MAP;
			case 0x8515 -> TEXTURE_CUBE_MAP_POSITIVE_X;
			case 0x8516 -> TEXTURE_CUBE_MAP_NEGATIVE_X;
			case 0x8517 -> TEXTURE_CUBE_MAP_POSITIVE_Y;
			case 0x8518 -> TEXTURE_CUBE_MAP_NEGATIVE_Y;
			case 0x8519 -> TEXTURE_CUBE_MAP_POSITIVE_Z;
			case 0x851a -> TEXTURE_CUBE_MAP_NEGATIVE_Z;
			case 0x851b -> PROXY_TEXTURE_CUBE_MAP;
			case 0x8c18 -> TEXTURE_1D_ARRAY;
			case 0x8c19 -> PROXY_TEXTURE_1D_ARRAY;
			case 0x8c1a -> TEXTURE_2D_ARRAY;
			case 0x8c1b -> PROXY_TEXTURE_2D_ARRAY;
			case 0x8c2a -> TEXTURE_BUFFER;
			case 0x8d41 -> RENDERBUFFER;
			case 0x9009 -> TEXTURE_CUBE_MAP_ARRAY;
			case 0x900b -> PROXY_TEXTURE_CUBE_MAP_ARRAY;
			case 0x9100 -> TEXTURE_2D_MULTISAMPLE;
			case 0x9101 -> PROXY_TEXTURE_2D_MULTISAMPLE;
			case 0x9102 -> TEXTURE_2D_MULTISAMPLE_ARRAY;
			case 0x9103 -> PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLTextureTarget");
		};
	}
}