package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLTraceMaskMESA implements GLEnum {
	TRACE_OPERATIONS_BIT_MESA(0x1),
	TRACE_PRIMITIVES_BIT_MESA(0x2),
	TRACE_ARRAYS_BIT_MESA(0x4),
	TRACE_TEXTURES_BIT_MESA(0x8),
	TRACE_PIXELS_BIT_MESA(0x10),
	TRACE_ERRORS_BIT_MESA(0x20),
	TRACE_ALL_BITS_MESA(0xffff);
	
	private final int glEnum;

	GLTraceMaskMESA(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLTraceMaskMESA> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLTraceMaskMESA fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> TRACE_OPERATIONS_BIT_MESA;
			case 0x2 -> TRACE_PRIMITIVES_BIT_MESA;
			case 0x4 -> TRACE_ARRAYS_BIT_MESA;
			case 0x8 -> TRACE_TEXTURES_BIT_MESA;
			case 0x10 -> TRACE_PIXELS_BIT_MESA;
			case 0x20 -> TRACE_ERRORS_BIT_MESA;
			case 0xffff -> TRACE_ALL_BITS_MESA;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLTraceMaskMESA");
		};
	}
}